from PIL import Image,ImageDraw
from ...utils import pill, git

_of = git.ImageCache()

class Weapon:
    def __init__(self,weapon,lang) -> None:
        self.weapon = weapon
        self.lang = lang
        self.CRIT_DMG = 0
        self.CRIT_RATE = 0
        
    async def add_substat(self):
        for substat in self.weapon.detail.substats:
            icon = await pill.get_icon_add(substat.prop_id, size=(25,25))
            self.background.alpha_composite(icon,(379,86))
            if str(substat.type) == "DigitType.PERCENT":
                val = f"{substat.value}%"
            else:
                val = str(substat.value)

            if substat.prop_id == "FIGHT_PROP_CRITICAL_HURT":
                self.CRIT_DMG += substat.value
            if substat.prop_id == "FIGHT_PROP_CRITICAL":
                self.CRIT_RATE += substat.value
            
            x = self.font_25.getlength(val)
            self.d.text((int(465-x/2),84), val, font= self.font_25, fill=(255,255,255,255))
            
    async def add_weapon_icon(self):
        icon = await pill.get_dowload_img(self.weapon.detail.icon.url, size=((180,180)))
        self.background.alpha_composite(icon,(43,0))
    
    async def add_stars(self):
        icon = await pill.get_stars(self.weapon.detail.rarity, light= True)
        self.background.alpha_composite(icon.resize((186,22)),(0,168))
    
    async def add_name(self):
        name = await pill.create_image_with_text(self.weapon.detail.name, 25, max_width=294, color=(255, 255, 255, 255))
        self.background.alpha_composite(name,(225,int(34-name.size[1]/2)))
    
    async def add_stats(self):
        lvl = f"{self.lang}: {self.weapon.level}/{self.weapon.max_level}"
        lvl_up = f"R{self.weapon.refinement}"
        base_stats = str(self.weapon.detail.mainstats.value)
        
        self.font_25 = await pill.get_font(25)
        
        self.d = ImageDraw.Draw(self.background)
        
        x = self.font_25.getlength(base_stats)
        self.d.text((int(311-x/2),84), base_stats, font= self.font_25, fill=(255,255,255,255))
        x = self.font_25.getlength(lvl_up)
        self.d.text((int(242-x/2),145), lvl_up, font= self.font_25, fill=(255,208,127,255))
        x = self.font_25.getlength(lvl)
        self.d.text((int(388-x/2),145), lvl, font= self.font_25, fill=(255,255,255,255))
    
    async def start(self):
        if self.weapon.detail.artifact_name_set != "":
            return {"tcv": 0, "img": Image.new("RGBA", (537, 161), (0,0,0,0))}
        
        self.background = await _of.weapon
        self.background = self.background.copy()
        
        await self.add_weapon_icon()
        await self.add_stats()
        await self.add_name()
        await self.add_stars()
        await self.add_substat()

        return {"tcv": float('{:.2f}'.format(self.CRIT_DMG + (self.CRIT_RATE*2))), "img": self.background}
        
        


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
import asyncio
from PIL import ImageDraw,Image
from ..utils import pill, git
from enkanetwork.enum import EquipmentsType
from .one import background, artifact, weapon, constant, skill, stat, prop
_of = git.ImageCache()
 
class Creat:
    def __init__(self,characters,lang,img,hide,uid,name) -> None:
        self.character = characters
        self.lang = lang["lvl"]
        self.img = img
        self.hide = hide
        self.uid = uid
        self.name = name
        
        
    
    async def creat_name_user(self):
        self.background_name_user = Image.new("RGBA", (249, 49), (0,0,0,0))
        font_20 = await pill.get_font(20)
        d = ImageDraw.Draw(self.background_name_user)
        d.text((0,0), self.name, font= font_20, fill=(255,255,255,150))
        if self.hide:
            d.text((0,25), "UID: Hide", font= font_20, fill=(255,255,255,150))
        else:
            d.text((0,25), f"UID: {self.uid}", font= font_20, fill=(255,255,255,150))
            
    
    async def collect_background(self):
        if self.img:
            self.background =  await background.Background(self.img, None).start()
        else:
            self.background = await background.Background(self.character.image.banner.url,self.character.element.value).start()
        
    async def collect_artifact(self):
        task_art = []
        task_prop = []
        ids_next = {"4": None,"2": None,"5": None,"1": None,"3": None}
        for key in filter(lambda x: x.type == EquipmentsType.ARTIFACT, self.character.equipments):
            ids = key.detail.icon.filename[-1:]
            ids_next[ids] = key
        
        for key in ids_next:
            task_art.append(artifact.Artifact(ids_next[key]).start())
            if not ids_next[key] is None:
                task_prop.append(prop.Prop(ids_next[key]).start())
        
        results = await asyncio.gather(*task_art, *task_prop)
        self.artifact = results[:len(task_art)]
        self.prop = results[len(task_art):]

    async def collect_sets(self):
        self.artifact_sets = await artifact.ArtifactSets(self.character.equipments).start()
    
    async def creat_name(self):
        self.background_name = Image.new("RGBA", (479, 149), (0,0,0,0))
        name = self.character.name
        level = f"{self.lang}: {self.character.level}/{self.character.max_level}"
        stars = await pill.get_stars(self.character.rarity)
        friends = str(self.character.friendship_level)
        font_25 = await pill.get_font(25)
        d = ImageDraw.Draw(self.background_name)
        x = font_25.getlength(name)
        d.text((int(468-x),0), name, font= font_25, fill=(255,255,255,255))
        x = font_25.getlength(level)
        d.text((int(468-x),40), level, font= font_25, fill=(255,255,255,255))
        friends_icon = await pill.get_icon_add("FRIENDS", size=(44,44))
        self.background_name.alpha_composite(stars.resize((101,29)),(378,120))
        self.background_name.alpha_composite(friends_icon,(428,76))
        x = font_25.getlength(friends)
        d.text((int(424-x),87), friends, font= font_25, fill=(255,255,255,255))
        
    async def creat_skill(self): 
        task = []
        self.background_skill = Image.new("RGBA", (97, 387), (0,0,0,0))
        for key in self.character.skills:
            task.append(skill.Skill(key,self.character.element.value).start())
        
        result = await asyncio.gather(*task)
        
        y = 0
        for key in result:
            self.background_skill.alpha_composite(key,(0,y))
            y += 146
            
        
    async def build(self):
        y = 239
        self.tcv = 0
        for key in self.artifact:
            self.background.alpha_composite(key["img"],(941,y))
            y += 168
            self.tcv += key["tcv"]
        
        self.background.alpha_composite(self.artifact_sets,(941,1100))
        self.background.alpha_composite(self.weapon["img"],(941,16))
        self.background.alpha_composite(self.background_name,(451,426))
        self.background.alpha_composite(self.background_constant, (10,494))
        self.background.alpha_composite(self.background_skill,(809,656))
        self.background.alpha_composite(self.background_stat, (11,16))
        self.background.alpha_composite(self.background_name_user,(11,1216))
        self.tcv += self.weapon["tcv"]
        await self.add_tcr()
    
    async def add_tcr(self):
        background_tcv = await _of.tcv
        background_tcv = background_tcv.copy()
        font_23 = await pill.get_font(23)
        d = ImageDraw.Draw(background_tcv)
        self.tcv = float('{:.2f}'.format(self.tcv))
        x = font_23.getlength(f"{self.tcv}TCV")
        d.text((int(76-x/2),4), f"{self.tcv}TCV", font= font_23, fill=(255,255,255,255))
        
        self.background.alpha_composite(background_tcv,(660,506))
    
    async def add_logo(self):
        logo = await _of.logo
        self.background.alpha_composite(logo, (1163,1216))
    
    async def collect_weapon(self):
        self.weapon = await weapon.Weapon(self.character.equipments[-1], self.lang).start()
    
    async def collect_constant(self):
        self.background_constant = Image.new("RGBA", (108, 672), (0,0,0,0))
        task = []
        y = 0
        for key in self.character.constellations:
            task.append(constant.Constant(key,self.character.element.value).start())
        
        result = await asyncio.gather(*task)
        
        for key in result:
            self.background_constant.alpha_composite(key,(0,y))
            y += 112
        
        
    async def collect_stat(self):
        self.background_stat = await stat.Stat(self.character.stats, self.prop, self.character.element.value).start()
        
    async def start(self):       
        tasks = [self.collect_artifact(), self.creat_name_user(),self.collect_background()]
        await asyncio.gather(*tasks)
        tasks = [
            self.collect_weapon(),
            self.creat_skill(),
            self.collect_sets(),
            self.collect_constant(),
            self.creat_name(),
            self.add_logo(),
            self.collect_stat()
        ]

        await asyncio.gather(*tasks)

        await self.build()

        return {
            "id": self.character.id,
            "name": self.character.name,
            "element": self.character.element.value,
            "rarity": self.character.rarity,
            "card": self.background
        }



import asyncio
from PIL import ImageDraw,Image,ImageChops
from ..utils import pill, git,diagram,options
from enkanetwork.enum import EquipmentsType
from .two import background, artifact, stat, skill
from .one import weapon,constant
from .akasha_rank import AkashaCreat
import random
_of = git.ImageCache()







class Creat:
    def __init__(self,characters,lang,img,hide,uid,name) -> None:
        self.character = characters
        self.lang = lang["lvl"]
        self.img = img
        self.hide = hide
        self.uid = uid
        self.name = name
        self.tcv = 0
        
    async def collect_background(self):
        if self.img:
            self.background =  await background.Background(self.img, None).start()
        else:
            self.background = await background.Background(self.character.image.banner.url,self.character.element.value).start()
        
    async def collect_weapon(self):
        self.weapon = await weapon.Weapon(self.character.equipments[-1], self.lang).start()
        self.weapon = self.weapon["img"].resize((351,129))

    async def collect_constant(self):
        self.background_constant = Image.new("RGBA", (108, 672), (0,0,0,0))
        task = []
        y = 0
        for key in self.character.constellations:
            task.append(constant.Constant(key,self.character.element.value).start())
        
        result = await asyncio.gather(*task)
        
        for key in result:
            self.background_constant.alpha_composite(key,(0,y))
            y += 112
        self.background_constant = self.background_constant.resize((54,342))
        
    async def collect_artifact(self):
        task_art = []
        y = 0
        
        ids_next = {"4": None,"2": None,"5": None,"1": None,"3": None}
        self.artifact_background = Image.new("RGBA", (457, 655), (0,0,0,0))
        
        for key in filter(lambda x: x.type == EquipmentsType.ARTIFACT, self.character.equipments):
            ids = key.detail.icon.filename[-1:]
            ids_next[ids] = key
        
        for key in ids_next:
            task_art.append(artifact.Artifact(ids_next[key],self.character.element.value).start())
        
        results = await asyncio.gather(*task_art)
        
        for key in results:
            self.artifact_background.alpha_composite(key["img"],(0,y))
            y += 133
            self.tcv += key["tcv"]
        
    async def collect_sets(self):
        self.artifact_sets = await artifact.ArtifactSets(self.character.equipments,self.character.element.value).start()
    
    async def creat_name(self):
        self.background_name = Image.new("RGBA", (350, 75), (0,0,0,0))
        name = self.character.name
        level = f"{self.lang}: {self.character.level}/{self.character.max_level}"
        stars = await pill.get_stars(self.character.rarity)
        friends = str(self.character.friendship_level)
        font_23 = await pill.get_font(23)
        font_15 = await pill.get_font(15)
        d = ImageDraw.Draw(self.background_name)
        x = font_23.getlength(name)
        d.text((int(349-x),0), name, font= font_23, fill=(255,255,255,255))
        
        x = font_15.getlength(level)
        d.text((int(349-x),31), level, font= font_15, fill=(255,255,255,255))
        
        friends_icon = await pill.get_icon_add("FRIENDS", size=(21,21))
        self.background_name.alpha_composite(stars.resize((101,29)),(202,46))
        self.background_name.alpha_composite(friends_icon,(328,50))
        x = font_15.getlength(friends)
        d.text((int(324-x),51), friends, font= font_15, fill=(255,255,255,255))
       
    async def add_logo(self):
        logo = await _of.logo
        self.background.alpha_composite(logo, (1108,710))
    
    async def creat_diagram(self):
        elementUp = True
        akashaElement = False
        data_user = []
        
        dataAkasha = await AkashaCreat().get_info_character(self.character.id)
        if dataAkasha != {}:
            dataAkasha  = dataAkasha[random.choice(list(dataAkasha.keys()))]['stats']
        dataAkasha_new = []
        
        for key in self.character.stats:
            if key[1].id in [40,41,42,43,44,45,46]:
                if elementUp:
                    key = max((x for x in self.character.stats if 40 <= x[1].id <= 46), key=lambda x: x[1].value)
                    elementUp = False
                    akashaElement = True
                else:
                    continue
            if key[1].value != 0 and key[0] in options.IconAddTrue:
                akasha_name = options.AkashaStats.get(key[0], None)
                if not akasha_name is None or akashaElement:
                    if akashaElement:
                        for keys in dataAkasha:
                            if "DMG" in keys and not "_CRITICAL" in keys:
                                akasha_name = keys
                                akashaElement = False
                                    
                    name = options.assets.get_hash_map(key[0])
                    
                    if "." in name:
                        name = name.replace(".",".\n")
                    else:
                        name = name.replace(" ","\n")
                    
                    try:
                        value = key[1].to_percentage()
                        procent = True
                    except:
                        value = key[1].to_rounded()
                        procent = False
                        
                    if dataAkasha != {}:
                        if procent:
                            dataAkasha_new.append({"name": name, "value": float('{:.2f}'.format(dataAkasha[akasha_name]))})
                        else:
                            dataAkasha_new.append({"name": name, "value": round(dataAkasha[akasha_name])})

                    data_user.append({"name": name, "value": value})
        
        if dataAkasha == {}:
            dataAkasha_new = [{'name': entry['name'], 'value': entry['value']} for entry in data_user]  
        
        self.diagram = await diagram.create_normalized_radial_chart(data_user,dataAkasha_new,self.character.element.value)

    async def creat_stat(self):
        self.stat_background = Image.new("RGBA", (519, 601), (0,0,0,0))
        result = await stat.Stat(self.character.stats, self.character.element.value).start()
        if len(result) <= 1:
            x = 0 
        x = int(519 / (len(result) - 1))
        position_line = 0
        for key in result:
            self.stat_background.alpha_composite(key["icon"], (0, position_line))
            position_line += x
    
    async def creat_skill(self): 
        task = []
        self.background_skill = Image.new("RGBA", (288, 81), (0,0,0,0))
        for key in self.character.skills:
            task.append(skill.Skill(key,self.character.element.value).start())
        
        result = await asyncio.gather(*task)
        
        x = 0
        for key in result:
            self.background_skill.alpha_composite(key,(x,0))
            x += 103
    
    async def add_tcr(self):
        background_tcv = await _of.tcv
        background_tcv = background_tcv.copy()
        font_23 = await pill.get_font(23)
        d = ImageDraw.Draw(background_tcv)
        self.tcv = float('{:.2f}'.format(self.tcv))
        x = font_23.getlength(f"{self.tcv}TCV")
        d.text((int(76-x/2),4), f"{self.tcv}TCV", font= font_23, fill=(255,255,255,255))
        
        self.background.alpha_composite(background_tcv.resize((99,23)),(1324,106))
    
    async def creat_name_user(self):
        self.background_name_user = Image.new("RGBA", (128, 32), (0,0,0,0))
        font_13 = await pill.get_font(13)
        d = ImageDraw.Draw(self.background_name_user)
        d.text((0,0), self.name, font= font_13, fill=(255,255,255,150))
        if self.hide:
            d.text((0,18), "UID: Hide", font= font_13, fill=(255,255,255,150))
        else:
            d.text((0,18), f"UID: {self.uid}", font= font_13, fill=(255,255,255,150))
    
    async def build(self):
        self.background.alpha_composite(self.weapon,(518,7))
        self.background.alpha_composite(self.background_constant,(1449,30))
        self.background.alpha_composite(self.artifact_background,(37,22))
        self.background.alpha_composite(self.artifact_sets,(32,696))
        self.background.alpha_composite(self.background_name,(1073,22))
        self.background.alpha_composite(self.diagram.resize((556,413)),(962 ,262))
        self.background.alpha_composite(self.stat_background,(519,155))
        self.background.alpha_composite(self.background_skill,(1096,155))
        self.background.alpha_composite(self.background_name_user,(1793,769))
        
        await self.add_tcr()
        await self.add_logo()
    
    async def start(self,snow):
        await asyncio.gather(self.collect_background(),self.collect_weapon(),self.collect_constant(),self.collect_artifact(),self.collect_sets(),self.creat_name(),
                             self.creat_name_user(),self.creat_diagram(),self.creat_stat(),self.creat_skill())
        
        await self.build()
        
        if snow:
            snow = await _of.snow
            snow_texture = await _of.snow_texture
            self.background = ImageChops.soft_light(self.background , snow_texture.convert("RGBA"))
        
        return {
            "id": self.character.id,
            "name": self.character.name,
            "element": self.character.element.value,
            "rarity": self.character.rarity,
            "card": self.background
        }


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
from enkanetwork import EnkaNetworkAPI, Assets
import random, aiohttp, json
from . import affixes
assets = Assets(lang="en")

async def fetch_json(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if response.status == 200:
                text_data = await response.text()
                json_data = json.loads(text_data)
                return json_data
            else:
                print(f"Error: {response.status}")
                return None

data_prop_json = affixes.data_prop_json
    
async def data_prop():
    global data_prop_json
    if data_prop_json is None:
        url = "https://raw.githubusercontent.com/EnkaNetwork/API-docs/master/store/affixes.json"
        data_prop_json = await fetch_json(url)
        
        return data_prop_json
    else:
        data_prop_json = None
        return None


AkashaStats = {
    "FIGHT_PROP_CRITICAL": "critRate",
    "FIGHT_PROP_CRITICAL_HURT": "critDMG",
    "FIGHT_PROP_CHARGE_EFFICIENCY": "energyRecharge",
    "FIGHT_PROP_ELEMENT_MASTERY":"elementalMastery",
    "FIGHT_PROP_MAX_HP": "maxHP",
    "FIGHT_PROP_CUR_ATTACK":"maxDEF",
    "FIGHT_PROP_CUR_DEFENSE":"maxATK"
}


color_artifact_up = {
    0: (255,255,255,255),
    1: (255,142,142,255),
    2: (255,214,142,255),
    3: (145,255,142,255),
    4: (142,255,240,255) #(235,142,255,255)
}

async def set_assets(lang):
    global assets
    assets = Assets(lang=lang)

async def get_charter_id(data):
    data = [value.strip() for value in data.split(',') if value.strip()]

    data = [value for value in data if value.isdigit()]
    
    if data == []:
        return None
    return data

async def get_info_enka(uid,USER_AGENT,lang):
    async with EnkaNetworkAPI(user_agent = USER_AGENT, lang=lang) as client:
        result = await client.fetch_user_by_uid(uid)
        if result.characters:
            return result
        else:
            return None

async def get_uid(uids):
    if type(uids) == int or type(uids) == str:
        return str(uids).replace(' ', '').split(",")[0]
    else:
        return None
        

async def get_character_art(character_art):
    processed_dict = {}
    for key, value in character_art.items():
        if isinstance(value, list):
            processed_dict[key] = random.choice(value)
        else:
            processed_dict[key] = value

    return processed_dict

stat_perc = {3, 6, 9, 11, 12, 20, 21, 22, 23, 24, 25, 26, 27, 29, 30, 40, 41, 42, 43, 44, 45, 46, 47, 50, 51, 52, 53, 54, 55, 56, 3002, 3004, 3005, 3007, 3008, 3009, 3010, 3011, 3012, 3013, 3014, 3015, 3016, 3017, 3018, 3019, 3020, 3021, 3024}
IconAddTrue = ["FIGHT_PROP_PHYSICAL_ADD_HURT","FIGHT_PROP_HEAL_ADD","FIGHT_PROP_GRASS_ADD_HURT","FIGHT_PROP_FIRE_ADD_HURT","FIGHT_PROP_MAX_HP","FIGHT_PROP_CUR_ATTACK","FIGHT_PROP_CUR_DEFENSE","FIGHT_PROP_ELEMENT_MASTERY","FIGHT_PROP_CRITICAL","FIGHT_PROP_CRITICAL_HURT","FIGHT_PROP_CHARGE_EFFICIENCY","FIGHT_PROP_ELEC_ADD_HURT","FIGHT_PROP_ROCK_ADD_HURT","FIGHT_PROP_ICE_ADD_HURT","FIGHT_PROP_WIND_ADD_HURT","FIGHT_PROP_WATER_ADD_HURT"]
dopStatAtribute = {"FIGHT_PROP_MAX_HP": "BASE_HP", "FIGHT_PROP_CUR_ATTACK":"FIGHT_PROP_BASE_ATTACK","FIGHT_PROP_CUR_DEFENSE":"FIGHT_PROP_BASE_DEFENSE"}


async def check_settings(settings):
    allowed_keys = {
        "get_characters": True,
        "get_generate": True, 
        "add_characters": True,
        "add_generate": True,
        "delete_all_pickle": True, 
        "delete_characters_pickle": True,
        "delete_generate_pickle": True,
        "delete_user_characters": True,
        "delete_user_generate": True,
        "size": True,
        "auto_clear": True 
    }

    for key in allowed_keys:
        if key not in settings:
            settings[key] = False

    return settings


from PIL import ImageDraw,Image
from ...utils import pill, git, options
from enkanetwork.enum import EquipmentsType
import asyncio
from collections import defaultdict

_of = git.ImageCache()

class ArtifactSets:
    def __init__(self,artifact) -> None:
        self.artifact = artifact
    
    async def start(self):
        background =  await _of.sets
        background = background.copy()
        count = {}
        for item in filter(lambda x: x.type == EquipmentsType.ARTIFACT, self.artifact):
            if item.detail.artifact_name_set in count:
                count[item.detail.artifact_name_set] += 1
            else:
                count[item.detail.artifact_name_set] = 1

        set_name = {item: occurrences for item, occurrences in count.items() if occurrences > 1}
        position = [(275,31)]
        if len(set_name) == 2:
            position = [(275,14),(275,52)]
            
        font_20 = await pill.get_font(20)
        d = ImageDraw.Draw(background)
        
               
        for i,key in enumerate(set_name):
            name = await pill.create_image_with_text(key, 20, max_width=316, color=(133, 255, 142, 255))
            value = str(set_name[key])
            x = font_20.getlength(value)
            d.text((int(508-x),position[i][1]), value, font= font_20, fill=(255,255,255,255))
            background.alpha_composite(name,(int(275-name.size[0]/2),position[i][1]))
        
        return background


class Artifact:
    def __init__(self, artifact) -> None:
        self.artifact = artifact
        self.CRIT_DMG = 0
        self.CRIT_RATE = 0
        self.props = {}
        self.prop_value = {}
        self.prop_id_counts = defaultdict(int)
        
        self.add_icons = False
    
    async def creat_prop_id_count(self):
        if self.prop_id_counts == {}:
            for p in self.artifact.props:
                self.prop_id_counts[p.prop_id] += 1
        
    async def creat_artifact_up(self, sub):
        countProc = self.prop_id_counts.get(sub.prop_id, 0)
        color = options.color_artifact_up.get(countProc, (235, 142, 255, 255))
        countProc = f"+{countProc}"

        icon, bg_up, font_17 = await asyncio.gather(
            pill.get_icon_add(sub.prop_id, size=(23, 24)),
            _of.artifact_up,
            pill.get_font(17)
        )
        icon = await pill.apply_opacity(icon, opacity=0.5)

        bg = bg_up.copy()
        bg.alpha_composite(icon, (4, 1))

        d = ImageDraw.Draw(bg)
        x = font_17.getlength(countProc)
        d.text((int(14 - x / 2), 4), countProc, font=font_17, fill=(0, 0, 0, 255))
        d.text((int(15 - x / 2), 3), countProc, font=font_17, fill=color)
                           
        return bg
    
    async def add_cv(self):
        cv = f"{float('{:.2f}'.format(self.CRIT_DMG + (self.CRIT_RATE*2)))}CV"
        d = ImageDraw.Draw(self.background)
        x = self.font_20.getlength(cv)
        d.text((int(260-x/2),12), cv, font= self.font_20, fill=(255,255,255,255))
        
    async def add_dmg_and_rate(self,prop_id,value):
        if prop_id == "FIGHT_PROP_CRITICAL_HURT":
            self.CRIT_DMG += value
        if prop_id == "FIGHT_PROP_CRITICAL":
            self.CRIT_RATE += value
    
    async def add_icon(self):
        
        background = Image.new("RGBA", (537, 161), (0,0,0,0))
        background_two = Image.new("RGBA", (537, 161), (0,0,0,0))
        icon, mask= await asyncio.gather(pill.get_dowload_img(self.artifact.detail.icon.url, size=((201,201))), _of.mask_artifact)
        background.alpha_composite(icon,(-38,-20))
        background_two.paste(background.convert("RGBA"),(0,0),mask.convert("L"))
        
        self.background.alpha_composite(background_two)
    
    async def add_main_info(self):
        icon,font_30,stars = await asyncio.gather(pill.get_icon_add(self.artifact.detail.mainstats.prop_id, size=(39,39)),pill.get_font(30),pill.get_stars(self.artifact.detail.rarity))
        self.background.alpha_composite(icon,(133,17))
        
        level = f"+{self.artifact.level}"
        if str(self.artifact.detail.mainstats.type) == "DigitType.PERCENT":
            val = f"{self.artifact.detail.mainstats.value}%"
        else:
            val = str(self.artifact.detail.mainstats.value)

        d = ImageDraw.Draw(self.background)
        x = font_30.getlength(val)
        d.text((int(172-x),66), val, font= font_30, fill=(0,0,0,255))
        d.text((int(173-x),68), val, font= font_30, fill=(255,255,255,255))
        
        x = self.font_20.getlength(level)
        d.text((int(149-x/2),109), level, font= self.font_20, fill=(255,211,91,255))
        self.background.alpha_composite(stars.resize((101,29)),(78,132))

        await self.add_dmg_and_rate(self.artifact.detail.mainstats.prop_id,self.artifact.detail.mainstats.value)
        
    async def add_sub_icon(self,key,x,y,z,d,font_25,i):
        if str(key.type) == "DigitType.PERCENT":
            val = f"{key.value}%"
        else:
            val = str(key.value)
                
        icon, icon_up, _ = await asyncio.gather(pill.get_icon_add(key.prop_id, size=(33,33)), self.creat_artifact_up(key), self.add_dmg_and_rate(key.prop_id,key.value))
        self.background.alpha_composite(icon,(x,y))
        d.text((x+40,y+3), val, font= font_25, fill=(255,255,255,255))
                    
        self.background.alpha_composite(icon_up,(z,10))

    async def add_sub_info(self):
        x,y,z = 203,49,331
        font_25 = await pill.get_font(25)
        d = ImageDraw.Draw(self.background)
        
        i = 0
        task = []
        for key in self.artifact.detail.substats:
            task.append(self.add_sub_icon(key,x,y,z,d,font_25,i))
            x = 369
            if i == 1:
                x,y = 203,103
                
            z += 38
            i += 1
        await asyncio.gather(*task)      
        
    async def start(self):
        if self.artifact is None:
            return {"tcv": 0, "img": await _of.artifact_bg_none} 
        self.font_20,self.background,_ = await asyncio.gather(pill.get_font(20),_of.artifact_bg,self.creat_prop_id_count())
        self.background = self.background.copy()
        await self.add_icon()
        
        task = [self.add_sub_info(),self.add_main_info()]
               
        await asyncio.gather(*task)
        await self.add_cv()
        
        

        return {"tcv": float('{:.2f}'.format(self.CRIT_DMG + (self.CRIT_RATE*2))), "img": self.background}


import asyncio
import random
from PIL import ImageDraw,Image,ImageChops
from ..utils import pill, git, options

_of = git.ImageCache()

class ProfileCard:
    def __init__(self,profile,lang,img,hide,uid,background) -> None:
        self.profile = profile
        self.lang = lang
        self.img = img
        self.hide = hide
        self.uid = uid
        self.background = background
    
    
    async def creat_background(self):
        self.background_profile = Image.new("RGBA", (828, 1078), (0,0,0,0))
        
        maska,frame = await asyncio.gather(_of.maska_prof_bg,_of.frame_profile)
        if self.background is None:
            background_image = random.choice([await _of.bg_1, await  _of.bg_2, await  _of.bg_3])
            background_shadow = Image.new("RGBA", (828, 1078), (0,0,0,50))
        else:
            background_image = await pill.get_dowload_img(self.background)
            background_image = await pill.get_centr_honkai_art((828,1078),background_image)
            background_shadow = Image.new("RGBA", (828, 1078), (0,0,0,150))
        background_image = background_image.convert("RGBA")
        background_image.alpha_composite(background_shadow)
        self.background_profile.paste(background_image,(0,0),maska.convert("L"))
        self.background_profile.alpha_composite(frame)

    async def creat_charter(self,key):
        
        charter_profile = Image.new("RGBA", (147, 211), (0,0,0,50))
        if str(key.id) in self.img:
            url_id = self.img[str(key.id)]
        else:
            url_id = key.icon.url
            
        splash,mask = await asyncio.gather(pill.get_dowload_img(url_id),_of.maska_character)
        splash = await pill.get_centr_honkai_art((147,211),splash)
        charter_profile.paste(splash,(0,0),mask.convert("L"))
        
        stars = options.assets.character(key.id)
        stars = await pill.get_stars(stars.rarity)
        name = await pill.create_image_with_text(key.name, 15, max_width=135, color=(255, 255, 255, 255)) 
        
        charter_profile.alpha_composite(stars.resize((85,25)),(31,0))
        charter_profile.alpha_composite(name,(int(74-name.size[0]/2),int(189-name.size[1]/2)))       

        return charter_profile
    
    async def get_charter(self):
        task = []
        for key in self.profile.characters_preview:
            task.append(self.creat_charter(key))
        
        self.charter = await asyncio.gather(*task)
    
    async def creat_avatar(self):
        self.background_profile_avatar = Image.new("RGBA", (625, 319), (0,0,0,0))
        background_avatar = Image.new("RGBA", (168, 168), (0,0,0,0))
        if self.profile.avatar.icon is None:
            avatar = "https://api.ambr.top/assets/UI/UI_AvatarIcon_Paimon.png"
        else:
            avatar = self.profile.avatar.icon.url
            
        avatar,font_20,maska,ab_ac,desc_frame = await asyncio.gather(pill.get_dowload_img(avatar, size= (168,168)),pill.get_font(20),_of.avatar_maska,_of.icons,_of.desc_frame)
        background_avatar.paste(avatar,(0,0),maska.convert("L"))
    
        self.background_profile_avatar.alpha_composite(background_avatar,(19,44))
        self.background_profile_avatar.alpha_composite(ab_ac,(170,183))
        
        d = ImageDraw.Draw(self.background_profile_avatar)
        
        level = f"{self.lang['lvl']}: {self.profile.level}"
        Wlevel = f"{self.lang['WL']}: {self.profile.world_level}"
        if self.hide:
            uid  = 'UID: Hide'
        else:
            uid = f"UID: {self.uid}"
        
        d.text((179,67), self.profile.nickname, font= font_20, fill=(255,255,255,255))
        d.text((179,139), level, font= font_20, fill=(255,255,255,255))
        d.text((179,103), Wlevel, font= font_20, fill=(255,255,255,255))
        d.text((13,0), uid, font= font_20, fill=(255,255,255,255))
        
        d.text((332,193), f"{self.profile.abyss_floor}-{self.profile.abyss_room}", font= font_20, fill=(255,255,255,255))
        d.text((220,193), str(self.profile.achievement), font= font_20, fill=(255,255,255,255))
        
        signature = await pill.create_image_with_text(self.profile.signature, 20, max_width=600, color=(255, 255, 255, 255))
        self.background_profile_avatar.alpha_composite(desc_frame,(0,228))
        self.background_profile_avatar.alpha_composite(signature,(int(320-signature.size[0]/2),250))
        
    
        
    
    async def build(self):
        self.background_profile.alpha_composite(self.background_profile_avatar,(23,19))
        x,y = 15,450  
        for i, key in enumerate(self.charter):
            self.background_profile.alpha_composite(key,(x,y))
            x += 168
            if i == 3:
                x = 15
                y += 237
        logo = await _of.logo
        self.background_profile.alpha_composite(logo,(18,978))
                
    async def start(self):
        
        await asyncio.gather(self.creat_background(), self.get_charter(), self.creat_avatar())
        await self.build()        
        return self.background_profile


# Copyright 2022 DEViantUa <t.me/deviant_ua>
# All rights reserved.
supportLang= {"en":"en", 
        "ru":"ru",
        "vi": "vi", 
        "th":"th",
        "pt":"pt",
        "kr":"ko",
        "jp":"ja",
        "zh":"zh-cn",
        "id":"id",
        "fr":"fr",
        "es":"es",
        "de":"de",
        "chs":"zh-cn",
        "cht":"zh-tw",
        "kh":"en"
    }

translationLang = {"en":{"lvl": "LVL", "AR":"AR", "WL":"WL", "AC": "Achievements", "AB": "Abyss"}, 
        "ru": {"lvl": "Уровень", "AR":"РП", "WL":"УМ", "AC": "Достижения", "AB": "Бездна"},
        "vi": {'lvl': 'Cấp độ ', 'AR': 'AR', 'WL': 'WL', 'AC': 'Thành tích ', 'AB': 'Vực thẳm'},
        "th": {'lvl': 'ระดับ ', 'AR': 'AR', 'WL': 'WL', 'AC': ' ความสำเร็จ ', 'AB': 'Abyss'},
        "pt": {'lvl': 'Nível ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Conquistas ', 'AB': 'Abismo'},
        "kr": {'lvl': '레벨 ', 'AR': 'AR', 'WL': 'WL', 'AC': '업적', 'AB': '어비스'},
        "jp": {'lvl': 'レベル ', 'AR': 'AR', 'WL': 'WL', 'AC': 'アチーブメント', 'AB': 'アビス'},
        "zh": {'lvl': '等级', 'AR': 'AR', 'WL': 'WL', 'AC': '成就总数', 'AB': '深境螺旋'},
        "id": {'lvl': 'Level ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Prestasi ', 'AB': ' Abyss'},
        "fr": {'lvl': 'Niveau ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Réalisations ', 'AB': ' Abîme'},
        "es": {'lvl': 'Nivel ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Logros ', 'AB': ' Abismo'},
        "de": {'lvl': 'Level ', 'AR': 'AR', 'WL': 'WL', 'AC': ' Erfolge ', 'AB': ' Abyss'},
        "chs": {'lvl': '等级', 'AR': 'AR', 'WL': 'WL', 'AC': '成就总数', 'AB': '深境螺旋'},
        "cht": {'lvl': '等級', 'AR': 'AR', 'WL': 'WL', 'AC': '成就總數', 'AB': '深境螺旋'},
    }



import asyncio
import aiofiles
import io
from PIL import Image
from enkanetwork import EnkaNetworkAPI
import os
import datetime
from .src.utils.options import get_charter_id, get_info_enka,get_character_art,get_uid,set_assets,check_settings
from .src.utils.git import change_Font
from .src.utils.pickle_cashe import PickleCache
from .src.utils.translation import translationLang, supportLang
from .src.modal import enkacardCread
from .enc_error import ENCardError
from .src.generator import teample_one, akasha_rank, profile_teample_one, teample_two,profile_teample_two

async def update():
    async with EnkaNetworkAPI(user_agent= "ENC Library: 3.0.0") as ena:
        await ena.update_assets()

async def save_card(uid, image_data, name):
    data = datetime.datetime.now().strftime("%d_%m_%Y %H_%M")
    path = os.getcwd()
    
    try:
        os.makedirs(f'{path}/EnkaCardImg/{uid}', exist_ok=True)
    except FileExistsError:
        pass
    
    file_name = f"{path}/EnkaCardImg/{uid}/{name}_{data}.png"
    
    async with aiofiles.open(file_name, 'wb') as file:
        if isinstance(image_data, Image.Image):
            img_bytes = io.BytesIO()
            image_data.save(img_bytes, format='PNG')
            await file.write(img_bytes.getvalue())


async def set_lang(lang):
    if lang != "kh":
        typelang = 0
        await set_assets(lang)
        lang = lang
        translateLang = translationLang[lang]
        await change_Font(0)
    else:
        typelang = 1
        await set_assets("en")
        lang = "en"
        translateLang = translationLang[lang]
        await change_Font(1)
    
    return typelang,lang,translateLang

class Akasha:
    def __init__(self, uid) -> None:
        self.uid = uid
    
    async def get_stats(self, chart_id = None):
        rank = await akasha_rank.AkashaCreat([],0,0,self.uid).get_rank_akasha()
        if not chart_id is None:
            for key in rank:
                if key['id'] == chart_id:
                    return enkacardCread.AkashaRank(akasha = [key])
            return None
        
        return enkacardCread.AkashaRank(akasha = rank)
    
    async def refresh(self, prints = True):
        result = await akasha_rank.AkashaCreat([],0,0,self.uid).update()
        if prints:
            print(result["data"]["message"])
    
    async def start(self, card, teample):
        rank = await self.get_stats(card["id"])
        if not rank is None:
            card["card"] = await akasha_rank.AkashaCreat(card["card"],teample,rank.akasha[0],self.uid).start()
        
        return card  

class ENC:
    def __init__(self,lang = "en", uid = None, character_art = None,
            character_id = None, hide_uid = False, save = False, pickle = None, agent = "Library: 3.0.0") :
        
        self.character_ids = []
        self.character_name = []
        self.USER_AGENT = f"ENC {agent}"
        self.lang  = lang
        self.save = save
        self.img = None

        self.print_size = None
        
        self.pickle = pickle
        self.hide_uid = hide_uid
        self.character_id = character_id
        self.uid = uid
        self.character_art = character_art
        
        self.pickle_class = PickleCache(self.uid)
        
    async def __aenter__(self):
        self.uid = await get_uid(self.uid)
        
        if self.uid is None:
            raise ENCardError(5,"The UIDS parameter must be a number or a string. To pass multiple UIDs, separate them with commas.\nExample: uids = 55363")
        if self.lang in supportLang:
            self.typelang,self.lang,self.translateLang = await set_lang(self.lang)
        else:
            self.lang = "en"
        
        self.enc = await get_info_enka(self.uid,self.USER_AGENT,self.lang)

        if self.enc is None:
            raise ENCardError(1001, "Enable display of the showcase in the game or add characters there")
        
        if self.character_id:
            self.character_id = await get_charter_id(self.character_id)
        
        if self.character_art:
            if not isinstance(self.character_art, dict):
                raise ENCardError(4,"The character_art parameter must be a dictionary, where the key is the name of the character, and the parameter is an image.\nExample: character_art = {'1235': 'img.png', '1235': ['img.png','http.../img2.png']} or {'123596': 'img.png', '123854': 'http.../img2.png', ...}")
            else:
                self.character_art = await get_character_art(self.character_art)
        
        if not self.pickle is None:
            if type(self.pickle) != dict:
                 raise ENCardError(80, "Pickle argument must be in dict format")

            self.pickle = await check_settings(self.pickle)
            
            self.enc.characters = await self.pickle_class.setting_charters(self.pickle,self.enc.characters)
            
            if self.pickle["size"]:
                self.print_size = await self.pickle_class.size_pickle()
        else:
            self.pickle = await check_settings({})
                
        return self

    async def __aexit__(self, *args):
        pass
       
    def sorting(self,result):
        enc_card = {"info": {
            "uid": self.uid,
            "lang": self.lang,
            "save": self.save
            },
            "card": result, 
            "character_id": self.character_ids,
            "character_name": self.character_name,
            "pickle_size": self.print_size
        }
                
        return enkacardCread.EnkaCard(**enc_card)
    
    async def creat(self, template = 1, akasha = False, snow = False):
        template = int(template)
        task = []
        task_save = []
        
        generator = []
        gen_tools = []
        
        if self.pickle["get_generate"]:
            generator = await self.pickle_class.get_generator(template)
                        
        if not template in [1,2]:
            template = 1
                
        for key in self.enc.characters:
            self.character_ids.append(key.id)
            self.character_name.append(key.name)

            if self.character_id:
                if not str(key.id) in self.character_id:
                    continue  
                                     
            if key.id in generator:
                if not generator.get(key.id,None) is None:
                    gen_tools.append(generator.get(key.id))
                continue
            
            art = None
            if self.character_art:
                if str(key.id) in self.character_art:
                    art = self.character_art[str(key.id)]
            
            if template == 1:
                task.append(teample_one.Creat(key,self.translateLang,art,self.hide_uid,self.uid,self.enc.player.nickname).start())
            else:
                task.append(teample_two.Creat(key,self.translateLang,art,self.hide_uid,self.uid,self.enc.player.nickname).start(snow))
                
        
        
        result = await asyncio.gather(*task)
        
        if not self.pickle is None:
            if self.pickle["add_generate"] and  result != []:
                await self.pickle_class.add_generator(template,result)
        
        if gen_tools != []:
            result = result + gen_tools

        if akasha:
            akasha_result = []
            for key in result:
                akasha_result.append(Akasha(self.uid).start(key, template))
            result = await asyncio.gather(*akasha_result)
        
        if self.save:
            for key in result:
                task_save.append(save_card(self.uid,key["card"],key["id"]))
        
            await asyncio.gather(*task_save)
        
        return self.sorting(result)

    async def profile(self,teamplate = 1, card = False,background = None):
        
        for key in self.enc.characters:
            self.character_ids.append(key.id)
            self.character_name.append(key.name)
            
        data = {
            "player":{
                "name":self.enc.player.nickname,
                "uid": self.uid,
                "lang": self.lang,
                "achievement": self.enc.player.achievement,
                "level": self.enc.player.level,
                "world_level": self.enc.player.world_level,
                "abyss": f"{self.enc.player.abyss_floor}-{self.enc.player.abyss_room}",
                "avatar": None,
            },
            "characters":{
                "count": len(self.enc.characters),
                "character_name": self.character_name,
                "character_id": self.character_ids,
            },
            "card": None
        }
        
        if self.enc.player.avatar.icon is None:
            data["player"]["avatar"] = "https://api.ambr.top/assets/UI/UI_AvatarIcon_Paimon.png"
        else:
            data["player"]["avatar"] = self.enc.player.avatar.icon.url

        if card:
            if int(teamplate) == 1:
                data["card"] = await profile_teample_one.ProfileCard(self.enc.player,self.translateLang,self.character_art,self.hide_uid,self.uid,background).start()
            elif int(teamplate) == 2:
                data["card"] = await profile_teample_two.ProfileCard(self.enc.player,self.translateLang,self.character_art,self.hide_uid,self.uid,background).start()
            if self.save:
                await save_card(self.uid, data["card"], "profile")
        
        return enkacardCread.Profile(**data)




# Copyright 2022 DEViantUa <t.me/deviant_ua>
# All rights reserved.
class EnkaNetworkCardError(BaseException):
    def __init__(self, code, message):
        super().__init__(f"[{code}] {message}")
        self.code = code
        self.message = message

class ENCardError(EnkaNetworkCardError):
    pass


import os
from os import PathLike
from zlib import decompress, compress
from pickle import loads
from pathlib import Path
from pickletools import optimize
import time

try:
    from cloudpickle import dumps
except ImportError:
    from pickle import dumps

MISSING = object()
pickle_name = [
    "data_characters","data_card"
]
def save_pkz(obj, file, *, protocol=5, compresslevel=9):
    if isinstance(file, int):
        with open(file, 'wb') as file:
            save_pkz(obj, file, protocol=protocol, compresslevel=compresslevel)
    elif isinstance(file, (str, bytes, PathLike)):
        path = Path(file).expanduser()
        temp = Path(f"{file}~").expanduser()
        
        with open(temp, 'wb') as file:
            save_pkz(obj, file, protocol=protocol, compresslevel=compresslevel)
        
        temp.replace(path)
    elif any(
        callable(getattr(file, name, None))
        for name in ['read', 'write']
    ):
        file.write(
            compress(
                optimize(
                    dumps(
                        obj,
                        protocol=protocol,
                    ),
                ),
                level=compresslevel,
            ),
        )
    else:
        raise TypeError(
            'expected int, str, bytes, path-like or file-like object, not'
            f" {file.__class__.__name__}"
        )


def load_pkz(
    file,
    default=MISSING,
    *,
    default_factory=MISSING,
    protocol=5,
    compresslevel=9,
):
    if isinstance(file, int):
        with open(file, 'rb') as file:
            return load_pkz(
                file,
                default,
                default_factory=default_factory,
                protocol=protocol,
                compresslevel=compresslevel,
            )
    elif isinstance(file, (str, bytes, PathLike)):
        path = Path(file).expanduser()
        temp = Path(f"{file}~").expanduser()
        
        if temp.exists():
            if path.exists():
                temp.unlink()
            else:
                temp.replace(path)
        elif not path.exists():
            if default is MISSING:
                if default_factory is MISSING:
                    raise FileNotFoundError(path)
                else:
                    obj = default_factory()
            else:
                obj = default
            
            save_pkz(obj, path, protocol=protocol, compresslevel=compresslevel)
        
        with open(path, 'rb') as file:
            return load_pkz(
                file,
                default,
                default_factory=default_factory,
                protocol=protocol,
                compresslevel=compresslevel,
            )
    elif any(
        callable(getattr(file, name, None))
        for name in ['read', 'write']
    ):
        return loads(decompress(file.read()))
    else:
        raise TypeError(
            'expected int, str, bytes, path-like or file-like object, not'
            f" {file.__class__.__name__}"
        )


from collections import UserDict

async def get_file_size(file_size_bytes):
    if file_size_bytes < 1024:
        return f"{file_size_bytes} Byte"
    elif 1024 <= file_size_bytes < 1024 * 1024:
        return f"{file_size_bytes / 1024:.2f} KB ({file_size_bytes} Byte)"
    elif 1024 * 1024 <= file_size_bytes < 1024 * 1024 * 1024:
        return f"{file_size_bytes / (1024 * 1024):.2f} MB ({file_size_bytes} Byte)"
    else:
        return f"{file_size_bytes / (1024 * 1024 * 1024):.2f} GB ({file_size_bytes} Byte)"

class PickleCache(UserDict):
    def __init__(self, uid) -> None:
        super().__init__()

        self.uid = uid
    
    def __getitem__(self, key):
        current_time = time.time()
        
        default = {'timestamp': current_time}
        
        try:
            value = super().__getitem__(key)
        except KeyError:
            value = load_pkz(Path(__file__).parent.parent / 'pickle_cashe' / f"{key}.pkz", default)
            
            super().__setitem__(key, value)
        
        if key == 'data_card':
            if current_time > value['timestamp'] + 5 * 60:
                self[key] = value = default
        
        return value
    
    def __setitem__(self, key, item):
        super().__setitem__(key, item)
        
        save_pkz(item, Path(__file__).parent.parent / 'pickle_cashe' / f"{key}.pkz")
    
    async def get_data(self, data):
        if self.uid in self['data_characters']:
            new_data = []
            true_id = []
            for keys in data:
                true_id.append(keys.id)
            
            for key in self['data_characters'][self.uid]:
                if not str(key) in str(true_id):
                    new_data.append(self['data_characters'][self.uid][key])
            return new_data
        
        return []
    
    async def add_data(self, data):
        self['data_characters'][self.uid] = {f"{key.id}": key for key in data}
        self['data_characters'] = self['data_characters']
    
    async def get_generator(self,teamplate):
        if not self.uid in self['data_card']:
            self['data_card'][self.uid] = {}
        if teamplate in self['data_card'][self.uid]:
            return self['data_card'][self.uid][teamplate]
        return []
            
    async def add_generator(self, teamplate,data):
        for key in data:
            if not self.uid in self['data_card']:
                self['data_card'][self.uid] = {}
            
            if not teamplate in self['data_card'][self.uid]:
                self['data_card'][self.uid][teamplate] = {}
                
            self['data_card'][self.uid][teamplate][key["id"]] = key
            
        self['data_card'] = self['data_card']
    
    async def setting_charters(self,pickles,data):
        
        if pickles["add_characters"]:
            await self.add_data(data)
            
        if pickles["get_characters"]:
            cash_pickle = await self.get_data(data)
            if cash_pickle != []:
                return data + cash_pickle
            
        return data
    
    async def size_pickle(self):
        data = []
        for key in pickle_name:
            file_path = Path(__file__).parent.parent / 'pickle_cashe' / f"{key}.pkz"
            if os.path.exists(file_path):
                file_size = os.path.getsize(file_path)
                text = f"File size '{key}' amounts to {await get_file_size(file_size)}"
                data.append({"name": key, "size":file_size , "text": text})
            else:
                data.append({"name": key, "size": 0 , "text": "File does not exist"})
        return data


from PIL import Image,ImageChops,ImageFilter
from ...utils import pill, git
import asyncio


_of = git.ImageCache()

class Background:
    def __init__(self,img,element) -> None:
        self.img = img
        self.element = element
        self.element_color = None
        
    async def get_element_color(self):
        self.element_color = pill.element_color.get(self.element, (149,107,5,255))
        self.background_grandient = Image.new("RGBA", (1950, 813), self.element_color)
        

    async def add_overlay(self):
        overlay = await _of.overlay_bg
        self.grandient = ImageChops.soft_light(self.grandient, overlay.convert("RGBA"))
        self.background_grandient = ImageChops.soft_light(self.grandient, overlay.convert("RGBA"))

    async def creat_grandient(self):
        if self.element is None:
            self.grandient = await pill.GradientGenerator(self.img).generate(1, 813)
            self.grandient = self.grandient.resize((1950, 813)).convert("RGBA")
        else:
            self.grandient = Image.new("RGBA", (1950, 813), self.element_color)
            
        userImages_opacity = await pill.apply_opacity(self.img, opacity=0.6)
        if self.element is None:
            self.grandient.alpha_composite(userImages_opacity,(1242,0))
            self.grandient = self.grandient.filter(ImageFilter.GaussianBlur(30))
        else:
            self.grandient.alpha_composite(userImages_opacity,(698,-75))
            self.grandient = self.grandient.filter(ImageFilter.GaussianBlur(20))
        await self.add_overlay()
        shadow = Image.new("RGBA", (1950, 813), (0,0,0,50))
        
        self.grandient.alpha_composite(shadow,(0,0))
        line = await _of.line_bg
        self.grandient.alpha_composite(line,(-17,0))
        
    
    async def add_mask(self):
        maska_bg = await _of.maska_bg
        maska_bg = maska_bg.convert("L")
        
        mask =  await _of.total_mask_bg
        mask = mask.convert("L")
        
        self.background = Image.new("RGBA", (1950, 813), (0,0,0,0))
        
        if self.element is None:
            self.background_grandient.alpha_composite(self.img,(1242,0))
        else:
            self.background_grandient.alpha_composite(self.img,(698,-75))
        self.background_grandient.paste(self.grandient,(0,0),maska_bg)
        self.background.paste(self.background_grandient,(0,0),mask)       
    
    async def creat_art(self):
        if self.element is None:
            self.img = await pill.get_centr_honkai_art((798,821),self.img)
    
    async def start(self):
        self.img = await pill.get_dowload_img(self.img)   
        if not self.element is None:
            await self.get_element_color()
        else:
            self.background_grandient = Image.new("RGBA", (1950, 813), (0,0,0,255))
        await self.creat_art()
        await self.creat_grandient()
        await self.add_mask()
                
        return self.background 


import sys
import os
import json

async def search():
    python_paths = sys.path

    enkanetwork_path = None

    for path in python_paths:
        enkanetwork_candidate = os.path.join(path, "enkanetwork")
        if os.path.exists(enkanetwork_candidate):
            enkanetwork_path = enkanetwork_candidate
            break

    if enkanetwork_path:
        assets_path = os.path.join(enkanetwork_path, "assets")

        if not os.path.exists(assets_path):
            os.makedirs(assets_path)

        return assets_path



from .enc_error import ENCardError
from .src.modal import enkaToolsModel
from .src.utils.enkanetwork_update import enka_update
import base64
from io import BytesIO
import aiohttp
from contextlib import AsyncExitStack
import magic

IMAGE_TYPES = {
    'image/jpeg',
    'image/png',
}

async def download_image(session, url, headers=None, allow_redirects=True, use_range=None, offset=0, size=None, **kwargs):
    if headers is None:
        headers = {}

    params = {
        'headers': headers,
        'allow_redirects': allow_redirects,
        **kwargs,
    }

    if use_range is None:
        async with session.head(url, **params) as response:
            if response.headers.get('accept-ranges') == 'bytes':
                use_range = True
            else:
                use_range = False

    if use_range:
        if size is None:
            if offset:
                headers['range'] = f"bytes={offset}-"
        else:
            headers['range'] = f"bytes={offset}-{offset + size - 1}"

    async with session.get(url, headers=headers, allow_redirects=allow_redirects, **kwargs) as response:
        if response.status != 416:
            response.raise_for_status()

            buffer = BytesIO()
            async for data in response.content.iter_any():
                length = len(data)

                if use_range:
                    if size is not None and offset + length > offset + size:
                        buffer.write(data[:offset + size - offset])
                        break
                    else:
                        buffer.write(data)
                else:
                    buffer.write(data)

            return buffer.getvalue()

async def get_mimetype(session, url, size=2048, allow_redirects=True, **kwargs):
    async with session:
        data = await download_image(session, url, size=size, allow_redirects=allow_redirects, **kwargs)
        return magic.Magic(mime=True).from_buffer(data)

async def is_valid(session, url, allow_redirects=True, **kwargs):
    async with session:
        try:
            async with session.head(url, allow_redirects=allow_redirects, **kwargs) as response:
                return response.status == 200
        except aiohttp.ClientError:
            return False


class Tools:
    def __init__(self) -> None:
        pass
    
    async def update_assets(self, path = None):
        await enka_update.dowload(path= path)

    async def image_to_base64(self,image, format = "png"):
        buffered = BytesIO()
        image.save(buffered, format= format)
        img_str = base64.b64encode(buffered.getvalue())
        
        return img_str
    
    async def get_link_image(self,img, api_key = None):
        if api_key is None:
            raise ENCardError(800,'Get the API key on the page: https://api.imgbb.com/ (Click on "Get Api Key")')
        
        async with aiohttp.ClientSession() as session:
            url = 'https://api.imgbb.com/1/upload'
            headers = {
                'Accept': 'application/json'
            }
            
            par = {
                'key': api_key,
                'image': await self.image_to_base64(img) if not isinstance(img, str) else img
            }
            async with session.post(url, data=par, headers=headers) as req:
                data = await req.json()
                
                return enkaToolsModel.EnkaCardLink(**data)    

    async def get_pixv_headers(self):
        return {"referer": "https://www.pixiv.net/"}
    
    async def is_valid_image(self, url,session=None,allow_redirects=True,strict=False,**kwargs):
        async with AsyncExitStack() as stack:
            if url is None:
                session, url = url,session
            
            if session is None:
                session = await stack.enter_async_context(aiohttp.ClientSession())
                
            async with session:
                try:
                    async with session.head(url, allow_redirects=allow_redirects, **kwargs) as response:
                        if response.status != 200 or not response.headers.get('content-length'):
                            return False

                        content_type = response.headers.get('content-type')

                        if content_type == 'application/octet-stream' or strict:
                            use_range = response.headers.get('accept-ranges') == 'bytes'
                            content_type = await get_mimetype(session, url, use_range=use_range, allow_redirects=allow_redirects, **kwargs)

                        return content_type in IMAGE_TYPES
                except aiohttp.ClientError:
                    return False


from PIL import ImageDraw,Image
from ...utils import pill, git, options
from enkanetwork.enum import EquipmentsType
import asyncio
from collections import defaultdict

_of = git.ImageCache()

class ArtifactSets:
    def __init__(self,artifact, element) -> None:
        self.artifact = artifact
        self.element = pill.element_color_text.get(element, (149,107,5,255))
        
    async def start(self):
        background = Image.new("RGBA", (457, 59), (0,0,0,0))
        bg_count = await _of.artifact_set_count
        count = {}
        for item in filter(lambda x: x.type == EquipmentsType.ARTIFACT, self.artifact):
            if item.detail.artifact_name_set in count:
                count[item.detail.artifact_name_set] += 1
            else:
                count[item.detail.artifact_name_set] = 1

        set_name = {item: occurrences for item, occurrences in count.items() if occurrences > 1}
        position = [(0,21)]
        if len(set_name) == 2:
            position = [(0,0),(431,35)]
            
        font_16 = await pill.get_font(16)
        
        for i,key in enumerate(set_name):
            bg_coun = bg_count.copy()
            name = await pill.create_image_with_text(key, 17, max_width=398, color=self.element)

            dd = ImageDraw.Draw(bg_coun)
            if set_name[key] > 5:
                value = 4
            elif set_name[key] == 3:
                value = 2
            else:
                value = set_name[key]
            dd.text((6,2), str(value), font= font_16, fill=self.element)
            background.alpha_composite(bg_coun,position[i])
            
            if i == 0:
                background.alpha_composite(name,(position[i][0]+ 33,position[i][1]+2))
            else:
                background.alpha_composite(name,(int(position[i][0] - name.size[0] - 5) ,position[i][1]+2))
                           
        return background
 
class Artifact:
    
    def __init__(self, artifact,element) -> None:
        self.artifact = artifact
        self.CRIT_DMG = 0
        self.CRIT_RATE = 0 
        self.element = element
            
    async def add_icon_artifact(self):
        background = Image.new("RGBA", (457, 123), (0,0,0,0))
        self.background_icon = Image.new("RGBA", (457, 123), (0,0,0,0))
        icon, mask= await asyncio.gather(pill.get_dowload_img(self.artifact.detail.icon.url, size=((203,199))), _of.artifact_mask)
        background.alpha_composite(icon,(-49,-45))
        self.background_icon.paste(background.convert("RGBA"),(0,0),mask.convert("L"))
            
            
    async def build(self):
        self.background.alpha_composite(self.background_icon)
        self.background.alpha_composite(self.background_m_stat)
        self.background.alpha_composite(self.background_s_stat)
        await self.add_cv()
        
    async def add_dmg_and_rate(self,prop_id,value):
        if prop_id == "FIGHT_PROP_CRITICAL_HURT":
            self.CRIT_DMG += value
        if prop_id == "FIGHT_PROP_CRITICAL":
            self.CRIT_RATE += value
    
    async def creat_main_stat(self):
        stat_frame = await _of.artifact_eff
        self.background_m_stat = Image.new("RGBA", (457, 123), (0,0,0,0))
        self.background_m_stat.alpha_composite(stat_frame)
        element = pill.element_color_text.get(self.element, (149,107,5,255))
        icon,font_30,stars = await asyncio.gather(pill.get_icon_add(self.artifact.detail.mainstats.prop_id, size=(25,25)),pill.get_font(30),pill.get_stars(self.artifact.detail.rarity))
        
        self.background_m_stat.alpha_composite(icon,(154,6))
        self.background_m_stat.alpha_composite(stars.resize((83,24)),(98,98))
        
        level = f"+{self.artifact.level}"
        if str(self.artifact.detail.mainstats.type) == "DigitType.PERCENT":
            val = f"{self.artifact.detail.mainstats.value}%"
        else:
            val = str(self.artifact.detail.mainstats.value)

        d = ImageDraw.Draw(self.background_m_stat)
        x = font_30.getlength(val)
        d.text((int(179-x),39), val, font= font_30, fill=(0,0,0,255))
        d.text((int(180-x),40), val, font= font_30, fill=(255,255,255,255))
        
        x = self.font_17.getlength(level)
        d.text((int(130-x/2),10), level, font= self.font_17, fill= element)
        self.background_m_stat.alpha_composite(stars.resize((101,29)),(78,132))

        await self.add_dmg_and_rate(self.artifact.detail.mainstats.prop_id,self.artifact.detail.mainstats.value)
    
    async def add_sub_icon(self,key,x,y,d):
        if str(key.type) == "DigitType.PERCENT":
            val = f"{key.value}%"
        else:
            val = str(key.value)
                
        icon, _ = await asyncio.gather(pill.get_icon_add(key.prop_id, size=(23,23)), self.add_dmg_and_rate(key.prop_id,key.value))
        self.background_s_stat.alpha_composite(icon,(x,y))
        d.text((x+40,y+2), val, font= self.font_17 , fill=(255,255,255,255))
    
    async def add_sub_info(self):
        self.background_s_stat = Image.new("RGBA", (457, 123), (0,0,0,0))
        
        x,y = 206,22
        d = ImageDraw.Draw(self.background_s_stat)
        i = 0
        task = []
        for key in self.artifact.detail.substats:
            task.append(self.add_sub_icon(key,x,y,d))
            x = 332
            if i == 1:
                x,y = 206,75
            i += 1
            
        await asyncio.gather(*task)  
    
    async def add_cv(self):
        cv = f"{float('{:.2f}'.format(self.CRIT_DMG + (self.CRIT_RATE*2)))}CV"
        d = ImageDraw.Draw(self.background)
        x = self.font_17.getlength(cv)
        d.text((int(140-x/2),77), cv, font= self.font_17, fill=(255,255,255,255))
    
    
    async def start(self):
        self.background = Image.new("RGBA", (457, 123), (0,0,0,50))
        
        if self.artifact is None:
            return {"tcv": 0, "img": self.background} 
        
        self.font_17 = await pill.get_font(17)
        
        await self.add_icon_artifact()
        await self.creat_main_stat()
        await self.add_sub_info()
        await self.build()
     
        return {"tcv": float('{:.2f}'.format(self.CRIT_DMG + (self.CRIT_RATE*2))), "img": self.background}


from PIL import Image,ImageDraw
from ...utils import pill, git, options
import asyncio
import json
import math
_of = git.ImageCache()


class Stat:
    def __init__(self,stats,element) -> None:
        self.stats = stats
        self.element_color = pill.element_color_text.get(element, (149,107,5,255))
        self.line = Image.new("RGBA", (26, 1), (255,255,255,255))
    
    
    async def get_substats(self):
        self.dop_value = {}
        stats_json = json.loads(self.stats.json())
        for key in stats_json:
            if key in options.dopStatAtribute:
                name_dop_value = options.dopStatAtribute.get(key)
                self.dop_value[key] = {"value": math.ceil(stats_json.get(name_dop_value)["value"])}
    
    async def creat_stats(self):
        task = []
        elementUp = True        
        for key in self.stats:
            if key[1].id in [40,41,42,43,44,45,46]:
                if elementUp:
                    key = max((x for x in self.stats if 40 <= x[1].id <= 46), key=lambda x: x[1].value)
                    elementUp = False
                else:
                    continue
            if key[1].value != 0 and key[0] in options.IconAddTrue:
                task.append(self.creat_line_stats(key))
            
        result = await asyncio.gather(*task)
        self.sorted_data = sorted(result, key=lambda x: x['id'] not in [2000, 2001, 2002])
    
    async def creat_line_stats(self,stats):
        background = Image.new("RGBA", (519, 47), (0,0,0,0))
        dop_value = 0
        name = options.assets.get_hash_map(stats[0])
        icon,name = await asyncio.gather(pill.get_icon_add(stats[0].replace("_CUR","").replace("_MAX", ""), size=(45,45)), pill.create_image_with_text(name, 18, max_width=155,  color=(255, 255, 255, 255)))
        try:
            value = stats[1].to_percentage_symbol()
        except:
            value = stats[1].to_rounded()
        if stats[0] in self.dop_value:
            dop_value = self.dop_value.get(stats[0], 0)
            up_valur = str(int(value - dop_value["value"]))
            
        
        background.alpha_composite(icon,(0,1))
        background.alpha_composite(name,(63,int(25-name.size[1]/2)))
        
        d = ImageDraw.Draw(background)
        xs = self.font_29.getlength(str(value))
        d.text((int(519-xs),0), str(value), font= self.font_29, fill=(255,255,255,255))
        if dop_value != 0:
            x = self.font_12.getlength(f"+{dop_value['value']}")
            d.text((int(519-x),30), f"+{dop_value['value']}", font= self.font_12, fill= self.element_color)
            xx = self.font_12.getlength(up_valur)
            d.text((int(519-x-xx-5),30), up_valur, font= self.font_12, fill=(255,255,255,255))
        
        size_x_line = int(519-name.size[0]-xs - 60-40)
        line = Image.new("RGBA", (size_x_line, 1), (255,255,255,50))
        background.alpha_composite(line, (63+name.size[0] + 20,22))
                
        return {"icon": background,"id": stats[1].id}
                            
    async def start(self):
        self.font_29,self.font_12 = await asyncio.gather(pill.get_font(29),pill.get_font(12))
        await self.get_substats()
        await self.creat_stats()
        
        return self.sorted_data


import asyncio
from PIL import ImageDraw,Image,ImageChops
from ..utils import pill, git, options
_of = git.ImageCache()

url_banner = "https://api.ambr.top/assets/UI/namecard/{id}.png"
url_splash = "https://enka.network/ui/{id}.png"

class ProfileCard:
    def __init__(self,profile,lang,img,hide,uid,background) -> None:
        self.profile = profile
        self.lang = lang
        self.img = img
        self.hide = hide
        self.uid = uid
        self.background = background
    
    async def get_background(self):
        position = (0,-63)
        if not self.background is None:
            banner = await pill.get_dowload_img(self.background)
            banner = await pill.get_centr_honkai_art((757,298),banner)
            position = (0,0)
        else:
            banner = url_banner.format(id = self.profile.namecard.banner.filename)
            banner = await pill.get_dowload_img(banner, size= (757,361))
        
        color = await pill.get_background_colors(banner, 15, common=True, radius=5, quality=800)
        
        background = Image.new("RGBA", (757, 717), color)
        background_banner = Image.new("RGBA", (757, 717), (0,0,0,0))
        background_banner.alpha_composite(banner,position)
        profile_bg_mask,overlay = await asyncio.gather(_of.profile_bg_mask,_of.overlay_profile)
        background.paste(background_banner,(0,0), profile_bg_mask.convert("L"))
        
        background = ImageChops.screen(background, overlay.convert("RGBA"))
        background_shadow = Image.new("RGBA", (757, 717), (0,0,0,150))
        background.alpha_composite(background_shadow)
        
        return background
        
    
    async def creat_avatar_info(self):
        background = Image.new("RGBA", (757, 156), (0,0,0,0))
        background_avatar = Image.new("RGBA", (120, 120), (0,0,0,0))
        if self.profile.avatar.icon is None:
            avatar = "https://api.ambr.top/assets/UI/UI_AvatarIcon_Paimon.png"
        else:
            avatar = self.profile.avatar.icon.url
        maska,ab_ac = await asyncio.gather(_of.avatar_mask, _of.ab_ac)
        avatar = await pill.get_dowload_img(avatar, size= (120,120))
        background_avatar.paste(avatar,(0,0),maska.convert("L"))
        background.alpha_composite(background_avatar,(320,7))
        background.alpha_composite(ab_ac,(274,23))
        
        font_15 = await pill.get_font(15)
        
        d = ImageDraw.Draw(background)
        
        level = f"{self.lang['lvl']}: {self.profile.level}"
        Wlevel = f"{self.lang['WL']}: {self.profile.world_level}"
        if self.hide:
            uid  = 'UID: Hide'
        else:
            uid = f"UID: {self.uid}"
        
        d.text((447,20), self.profile.nickname, font= font_15, fill=(255,255,255,255))
        d.text((447,44), level, font= font_15, fill=(255,255,255,255))
        d.text((447,68), Wlevel, font= font_15, fill=(255,255,255,255))
        d.text((447,92), uid, font= font_15, fill=(255,255,255,255))
        
        x = font_15.getlength(f"{self.profile.abyss_floor}-{self.profile.abyss_room}")
        d.text((int(268-x),88), f"{self.profile.abyss_floor}-{self.profile.abyss_room}", font= font_15, fill=(255,255,255,255))
        x = font_15.getlength(str(self.profile.achievement))
        d.text((int(268-x),35), str(self.profile.achievement), font= font_15, fill=(255,255,255,255))
        
        signature = await pill.create_image_with_text(self.profile.signature, 15, max_width=744, color=(255, 255, 255, 255))
        background.alpha_composite(signature,(int(380-signature.size[0]/2),135))
        
        return background
    
    async def creat_charter(self,key):
        background = Image.new("RGBA", (180, 263), (0,0,0,0))
        if str(key.id) in self.img:
            url = self.img[str(key.id)]
        else:
            if "Costume" in key.icon.filename:
                url_id = key.icon.filename.replace("UI_AvatarIcon","UI_Costume")
            else:
                url_id = key.icon.filename.replace("UI_AvatarIcon","UI_Gacha_AvatarImg")
            url = url_splash.format(id = url_id)
        splash,mask,shadow = await asyncio.gather(pill.get_dowload_img(url),_of.art_profile_mask,_of.shadow_art_profile)
        splash = await pill.get_centr_honkai_art((180,263),splash)
        background.paste(splash,(0,0),mask.convert("L"))
        background.alpha_composite(shadow)
        stars = options.assets.character(key.id)
        stars = await pill.get_stars(stars.rarity)
        background.alpha_composite(stars.resize((58,17)),(25,235))
        
        name = await pill.create_image_with_text(key.name, 15, max_width=112, color=(0, 0, 0, 255))
        background.alpha_composite(name,(29,int(219-name.size[1])))
        name = await pill.recolor_image(name,(255,255,255))
        background.alpha_composite(name,(30,int(219-name.size[1])))
        
        
        font_15 = await pill.get_font(15)
        
        d = ImageDraw.Draw(background)
        
        level = f"{self.lang['lvl']}: {key.level}"
        d.text((29,219), level, font= font_15, fill=(0,0,0,255))
        d.text((30,219), level, font= font_15, fill=(255,255,255,255))
        
        return background
     
    async def build(self):
        self.background.alpha_composite(self.avatar,(0,10))
        x,y = 9,170
        i = 0
        for key in self.charter:
            self.background.alpha_composite(key,(x,y))
            x += 185
            i += 1
            if i == 4:
                x = 9
                y += 277
    
    async def start(self):
        
        self.background,self.avatar = await asyncio.gather(self.get_background(),self.creat_avatar_info())
               
        task = []
        for key in self.profile.characters_preview:
            task.append(self.creat_charter(key))
        
        self.charter = await asyncio.gather(*task)
        await self.build()
        
        return self.background